@@ -26,7 +26,7 @@ void LatexParserTest::addRow(const char *name, LatexParserTest::TokenFilter filt
 	switch (filter) {
 	case FILTER_NEXTTOKEN:
 		for (int i = tokens.size() - 1; i >= 0; i--)
-			if (tokens[i].type == NW_IGNORED_TOKEN && tokens[i] != "{" && tokens[i] != "}" && tokens[i] != "[" && tokens[i] != "]")
+            if (tokens[i].type == NW_IGNORED_TOKEN && (tokens[i] != QString("{")) && (tokens[i] != QString("}")) && (tokens[i] != QString("[")) && (tokens[i] != QString("]")))
 				tokens.removeAt(i);
 		break;
 	case FILTER_NEXTWORD_WITH_COMMANDS:
@@ -64,25 +64,25 @@ void LatexParserTest::addComplexData(LatexParserTest::TokenFilter filter)
 	QTest::addColumn<QString >("str");
 	QTest::addColumn<QList<TestToken> >("tokens");
 
-	/*addRow("simple whitespace", filter,
-	        QList<TestToken>() << "abcde" << "    " << "fghik" << "\t" << "Mice");
-	    addRow("simple eow", filter,
-	           QList<TestToken>() << "abcde" << ";" << ":" << ";" << "fghik" << TestToken("##", NW_IGNORED_TOKEN) << "Mice" << TestToken("///", NW_IGNORED_TOKEN) << "\\\\" << TestToken("+++", NW_IGNORED_TOKEN) );//<< "axy" << TestToken("---", "-", LatexReader::NW_PUNCTATION)); "-" is now added to the word ...
-	    addRow("simple eow", filter,
-	           QList<TestToken>() << "abcde." << ";" << ":" << ";" << "fghik" << TestToken("##", NW_IGNORED_TOKEN) << "Mice" << TestToken("///", NW_IGNORED_TOKEN) << "\\\\" << TestToken("+++", NW_IGNORED_TOKEN) );//<< "axy" << TestToken("---", "-", LatexReader::NW_PUNCTATION));
-	    addRow("environment+comment",filter,
-	        QList<TestToken>() << "Test1234" << "\\begin" << "{" << env("environment") << "}" << "{" << "add" << "}" << "XYZ" << "!" << "!" << "!" << "\\command" << "%"     << "comment" << "\\COMMENT");
-	    addRow("some environments", filter,
-	        QList<TestToken>() << "\\newenvironment" << "{" << env("env") << "}" << " " << "\\begin" << "{" << env("env2") << "}" << " " << "\\end" << "{" << env("env3") << "}" << "  " << "\\renewenvironment" << "{" << env("env4") << "}");
-	    addRow("misc", filter, //was only for nextWord, other test will of course not ignore \\ignoreMe
-	        QList<TestToken>() << "hallo" << " " << "welt" << "\\section" << "{" << "text" << "}" << "     " << "\\begin" << "{" << env("I'mXnotXthere") << "}" << " *" << "g"  << "* " << "%"     << " " << "more" << " " << "\\comment");
-	    addRow("command as option", filter,
-	        QList<TestToken>() << "\\includegraphics" << "[" << option("ab.") << "\\linewidth" << "]" << "{" << "\\abc" << " " << option("dfdf") << "\\xyz" << "}" << "continue");
-	    addRow("command as option", filter,
-	           QList<TestToken>() << "\\includegraphics" << "[" << option("ab") << TestToken(":", NW_OPTION_PUNCTATION) << "\\linewidth" << "]" << "{" << "\\abc" << " " << option("dfdf") << "\\xyz" << "}" << "continue");
-	    addRow("comments", filter, QList<TestToken>() << "hallo" << " " << "welt" <<  "  " << "\\\\" << "normaltext" <<  "  " << TestToken("\\%",NW_IGNORED_TOKEN) << "!" << "!" << "!" << "stillNoComment" << "\\\\" << TestToken("\\%","%",NW_IGNORED_TOKEN) <<"  "<< "none" << "\\\\" << "%" << "comment" << "   " << "more" << " " << "comment");
-	    addRow("escaped characters", filter, QList<TestToken>() << "hallo" << TestToken("\\%",NW_IGNORED_TOKEN) << "abc");
-	    addRow("escaped characters", filter, QList<TestToken>() << "1234" << TestToken("\\%\\&\\_",NW_IGNORED_TOKEN)  << "567890");  */
+    addRow("simple whitespace", filter,
+           QList<TestToken>() << "abcde" << "    " << "fghik" << "\t" << "Mice");
+    addRow("simple eow", filter,
+           QList<TestToken>() << "abcde" << ";" << ":" << ";" << "fghik" << TestToken("##", NW_IGNORED_TOKEN) << "Mice" << TestToken("///", NW_IGNORED_TOKEN) << "\\\\" << TestToken("+++", NW_IGNORED_TOKEN) );//<< "axy" << TestToken("---", "-", LatexReader::NW_PUNCTATION)); "-" is now added to the word ...
+    addRow("simple eow", filter,
+           QList<TestToken>() << "abcde." << ";" << ":" << ";" << "fghik" << TestToken("##", NW_IGNORED_TOKEN) << "Mice" << TestToken("///", NW_IGNORED_TOKEN) << "\\\\" << TestToken("+++", NW_IGNORED_TOKEN) );//<< "axy" << TestToken("---", "-", LatexReader::NW_PUNCTATION));
+    addRow("environment+comment",filter,
+           QList<TestToken>() << "Test1234" << "\\begin" << "{" << env("environment") << "}" << "{" << "add" << "}" << "XYZ" << "!" << "!" << "!" << "\\command" << "%"     << "comment" << "\\COMMENT");
+    addRow("some environments", filter,
+           QList<TestToken>() << "\\newenvironment" << "{" << env("env") << "}" << " " << "\\begin" << "{" << env("env2") << "}" << " " << "\\end" << "{" << env("env3") << "}" << "  " << "\\renewenvironment" << "{" << env("env4") << "}");
+    addRow("misc", filter, //was only for nextWord, other test will of course not ignore \\ignoreMe
+           QList<TestToken>() << "hallo" << " " << "welt" << "\\section" << "{" << "text" << "}" << "     " << "\\begin" << "{" << env("I'mXnotXthere") << "}" << " *" << "g"  << "* " << "%"     << " " << "more" << " " << "\\comment");
+    addRow("command as option", filter,
+           QList<TestToken>() << "\\includegraphics" << "[" << option("ab.") << "\\linewidth" << "]" << "{" << "\\abc" << " " << option("dfdf") << "\\xyz" << "}" << "continue");
+    addRow("command as option", filter,
+           QList<TestToken>() << "\\includegraphics" << "[" << option("ab") << TestToken(":", NW_OPTION_PUNCTATION) << "\\linewidth" << "]" << "{" << "\\abc" << " " << option("dfdf") << "\\xyz" << "}" << "continue");
+    addRow("comments", filter, QList<TestToken>() << "hallo" << " " << "welt" <<  "  " << "\\\\" << "normaltext" <<  "  " << TestToken("\\%",NW_IGNORED_TOKEN) << "!" << "!" << "!" << "stillNoComment" << "\\\\" << TestToken("\\%","%",NW_IGNORED_TOKEN) <<"  "<< "none" << "\\\\" << "%" << "comment" << "   " << "more" << " " << "comment");
+    addRow("escaped characters", filter, QList<TestToken>() << "hallo" << TestToken("\\%",NW_IGNORED_TOKEN) << "abc");
+    addRow("escaped characters", filter, QList<TestToken>() << "1234" << TestToken("\\%\\&\\_",NW_IGNORED_TOKEN)  << "567890");
 	addRow("special characters", filter,
 	       QList<TestToken>() << "lösbar" << " " << TestToken("l\"osbar", QString::fromUtf8("lösbar"), LatexReader::NW_TEXT) << " " << TestToken("l\\\"osbar", QString::fromUtf8("lösbar"), LatexReader::NW_TEXT) << " " << TestToken("l\\\"{o}sbar", QString::fromUtf8("lösbar"), LatexReader::NW_TEXT) << " " << "örtlich" << " " << TestToken("\"ortlich", QString::fromUtf8("örtlich"), LatexReader::NW_TEXT) << " " << TestToken("\\\"ortlich", QString::fromUtf8("örtlich"), LatexReader::NW_TEXT) << " " << TestToken("\\\"{o}rtlich", QString::fromUtf8("örtlich"), LatexReader::NW_TEXT) );
 }
@@ -137,7 +137,8 @@ void LatexParserTest::nextToken_complex()
 		if (pos >= tokens.size()) {
 			QFAIL(QString("Found additional token: %1 at %2").arg(token).arg(startIndex).toLatin1().constData());
 		} else {
-			QVERIFY2(token == tokens[pos], QString("Invalid token: %1 at %2 expected %3").arg(token).arg(startIndex).arg(tokens[pos]).toLatin1().constData());
+            QString fnd=tokens[pos];
+            QVERIFY2(token == fnd, QString("Invalid token: %1 at %2 expected %3").arg(token).arg(startIndex).arg(tokens[pos]).toLatin1().constData());
 			QVERIFY2(startIndex == tokens[pos].position, QString("Invalid startIndex: %2 for %1").arg(token).arg(startIndex).toLatin1().constData());
 		}
 		pos++;
@@ -249,7 +250,7 @@ void LatexParserTest::nextWord_simple_data()
 	//QTest::newRow("sepchar8")  << "?oz\"yxdi?" << 1 << false << false << (int)LatexReader::NW_TEXT << 8 << "ozyxdi" << 1;  //invalid combinations in german.sty
 	//QTest::newRow("sepchar8")  << "?oz\"y?" << 1 << false << false << (int)LatexReader::NW_TEXT << 5 << "ozy" << 1; //invalid combinations in german.sty
 	QTest::newRow("word end")  << "?no\"<di?" << 1 << false << false << (int)LatexReader::NW_TEXT << 3 << "no" << 1;
-	QTest::newRow("word end")  << "?yi''di?" << 1 << false << false << (int)LatexReader::NW_TEXT << 3 << "yi" << 1;
+    QTest::newRow("word end2")  << "?yi''di?" << 1 << false << false << (int)LatexReader::NW_TEXT << 3 << "yi" << 1;
 	QTest::newRow("umlauts")  << "\"a\"o\"u\"A\"O\"U\\\"{a}\\\"{o}\\\"{u}\\\"{A}\\\"{O}\\\"{U}" << 0 << false << false << (int)LatexReader::NW_TEXT << 42 << (QString(QChar(0xE4)) + QString(QChar(0xF6)) + QString(QChar(0xFC)) + QString(QChar(0xC4)) + QString(QChar(0xD6)) + QString(QChar(0xDC)) + QString(QChar(0xE4)) + QString(QChar(0xF6)) + QString(QChar(0xFC)) + QString(QChar(0xC4)) + QString(QChar(0xD6)) + QString(QChar(0xDC))) << 0; //unicode to be independent from c++ character encoding
 }
 
@@ -304,68 +305,10 @@ void LatexParserTest::cutComment_simple()
 {
 	QFETCH(QString, in);
 	QFETCH(QString, out);
-	QString res = LatexParser::cutComment(in);
+    QString res = cutComment(in);
 	QEQUAL(res, out);
 }
 
-void LatexParserTest::test_findContext_data()
-{
-	QTest::addColumn<QString >("in");
-	QTest::addColumn<int>("pos");
-	QTest::addColumn<int>("out");
-
-	QTest::newRow("command") << "\\begin{test}" << 3 << 1;
-	QTest::newRow("content") << "\\begin{test}" << 8 << 3;
-	QTest::newRow("option") << "\\begin[abc]{test}" << 8 << 2;
-	QTest::newRow("content with option") << "\\begin[abc]{test}" << 13 << 3;
-	QTest::newRow("content with option2") << "\\begin[\\abc]{test}" << 14 << 3;
-	QTest::newRow("command with option") << "\\begin[abc]{test}" << 3 << 1;
-	QTest::newRow("nothing") << "\\begin{test}" << 0 << 0;
-}
-
-void LatexParserTest::test_findContext()
-{
-	QFETCH(QString, in);
-	QFETCH(int, pos);
-	QFETCH(int, out);
-	int res = LatexParser::getInstance().findContext(in, pos);
-	QEQUAL(res, out);
-}
-
-void LatexParserTest::test_findContext2_data()
-{
-	QTest::addColumn<QString >("in");
-	QTest::addColumn<int>("pos");
-	QTest::addColumn<int>("out");
-	QTest::addColumn<QString>("command");
-	QTest::addColumn<QString>("value");
-
-    QTest::newRow("command") << "\\begin{test}" << 3 << static_cast<int>(LatexParser::Command) << "\\begin" << "test";
-    QTest::newRow("content") << "\\begin{test}" << 8 << static_cast<int>(LatexParser::Environment) << "\\begin" << "test";
-    QTest::newRow("ref") << "\\ref{test}" << 8 << static_cast<int>(LatexParser::Reference) << "\\ref" << "test";
-    QTest::newRow("label") << "\\label{test}" << 8 << static_cast<int>(LatexParser::Label) << "\\label" << "test";
-    QTest::newRow("cite") << "\\cite{test}" << 8 << static_cast<int>(LatexParser::Citation) << "\\cite" << "test";
-    QTest::newRow("cite") << "\\cite{test}" << 3 << static_cast<int>(LatexParser::Command) << "\\cite" << "test";
-    QTest::newRow("abcd option") << "\\abcd{test}" << 7 << static_cast<int>(LatexParser::Option) << "\\abcd" << "test";
-    QTest::newRow("abcd option2") << "\\abcd[abc]{test}" << 12 << static_cast<int>(LatexParser::Option) << "\\abcd" << "test";
-    QTest::newRow("abcd option3") << "\\abcd[\\abc]{test}" << 12 << static_cast<int>(LatexParser::Option) << "\\abcd" << "test";
-}
-
-void LatexParserTest::test_findContext2()
-{
-	QFETCH(QString, in);
-	QFETCH(int, pos);
-	QFETCH(int, out);
-	QFETCH(QString, command);
-	QFETCH(QString, value);
-	QString cmd;
-	QString val;
-	LatexParser::ContextType res = LatexParser::getInstance().findContext(in, pos, cmd, val);
-    QEQUAL(static_cast<int>(res), out);
-	QEQUAL(cmd, command);
-	QEQUAL(val, value);
-}
-
 void LatexParserTest::test_resolveCommandOptions_data()
 {
 	QTest::addColumn<QString>("line");
@@ -395,7 +338,7 @@ void LatexParserTest::test_resolveCommandOptions()
 	QFETCH(bool, expectedComplete);
 	QStringList values;
 	QList<int> starts;
-	bool complete = LatexParser::resolveCommandOptions(line, column, values, &starts);
+    bool complete = resolveCommandOptions(line, column, values, &starts);
 	QEQUAL(values.length(), starts.length());
 	QEQUAL(values.join("|"), expectedValues.join("|"));
 	QVERIFY(starts == expectedStarts);
@@ -415,7 +358,7 @@ void LatexParserTest::test_findClosingBracket_data()
 	QTest::newRow("nested bracket") << "{a {nested} simple} text" << 0 << QChar('{') << QChar('}') << 18;
 	QTest::newRow("nested bracket2") << "{a {nested} simple} text" << 3 << QChar('{') << QChar('}') << 10;
 	QTest::newRow("nested bracket3") << "{a {nested} simple} text" << 16 << QChar('{') << QChar('}') << 18;
-	QTest::newRow("nested bracket3") << "{a {nested} simple} text" << 19 << QChar('{') << QChar('}') << -1;
+    QTest::newRow("nested bracket4") << "{a {nested} simple} text" << 19 << QChar('{') << QChar('}') << -1;
 	QTest::newRow("no bracket") << "No bracket in here" << 0 << QChar('{') << QChar('}') << -1;
 	QTest::newRow("no bracket2") << "No {proper} bracket in here" << 0 << QChar('{') << QChar('}') << -1;
 }
@@ -428,7 +371,27 @@ void LatexParserTest::test_findClosingBracket()
 	QFETCH(QChar, cc);
 	QFETCH(int, out);
 	int ret = findClosingBracket(line, start, oc, cc);
-	QEQUAL(ret, out);
+    QEQUAL(ret, out);
+}
+
+void LatexParserTest::test_interpretXArgs_data()
+{
+    QTest::addColumn<QString>("argument");
+    QTest::addColumn<QString>("result");
+
+    QTest::newRow("simple arguments") << "moO" << "{%<arg%>}[%<arg%>][%<arg%>]";
+    QTest::newRow("spaces") << " m o  O " << "{%<arg%>}[%<arg%>][%<arg%>]";
+    QTest::newRow("defined brackets") << "r()R&%d)(D][" << "(%<arg%>)&%<arg%>%)%<arg%>(]%<arg%>[";
+    QTest::newRow("braces") << "m{oO}" << "{%<arg%>}";
+    QTest::newRow("ignored characters") << "mstb+uo" << "{%<arg%>}[%<arg%>]";
+}
+
+void LatexParserTest::test_interpretXArgs()
+{
+    QFETCH(QString, argument);
+    QFETCH(QString, result);
+    QString out=interpretXArgs(argument);
+    QEQUAL(out,result);
 }
 
 #endif // QT_NO_DEBUG
