@@ -380,6 +380,54 @@ void LatexParsingTest::test_latexLexing_data() {
                                      << (Starts() << 0 << 11 << 12)
                                      << (Length() << 11 << 6 << 4)
                                      << (Levels() << 0 << 1 << 1 );
+    QTest::newRow("keyval label") << "\\againframe[label=abc]"
+                                  << (TTypes() << T::command << T::squareBracket << T::keyVal_key<< T::label)
+                                  << (STypes() << T::none << T::keyValArg << T::none<< T::keyVal_val)
+                                  << (Starts() << 0 << 11 << 12 << 18)
+                                  << (Length() << 11 << 11 << 5 << 3)
+                                  << (Levels() << 0 << 1 << 1 << 2 );
+    QTest::newRow("keyval label in braces") << "\\againframe[label={abc}]"
+                                  << (TTypes() << T::command << T::squareBracket << T::keyVal_key<< T::braces << T::label)
+                                  << (STypes() << T::none << T::keyValArg << T::none<< T::label << T::none)
+                                  << (Starts() << 0 << 11 << 12 << 18 <<19)
+                                  << (Length() << 11 << 13 << 5 << 5 << 3 )
+                                  << (Levels() << 0 << 1 << 1 << 3 << 3);
+    QTest::newRow("keyval length") << "\\mycommand{width=\\textwidth}"
+                                  << (TTypes() << T::command << T::braces << T::keyVal_key<< T::command)
+                                  << (STypes() << T::none << T::keyValArg << T::none<< T::width)
+                                  << (Starts() << 0 << 10 << 11 << 17)
+                                  << (Length() << 10 << 18 << 5 << 10)
+                                  << (Levels() << 0 << 1 << 1 << 2 );
+    QTest::newRow("keyval text") << "\\mycommand{title=abc}"
+                                           << (TTypes() << T::command << T::braces << T::keyVal_key<< T::word)
+                                           << (STypes() << T::none << T::keyValArg << T::none<< T::text)
+                                           << (Starts() << 0 << 10 << 11 << 17)
+                                           << (Length() << 10 << 11 << 5 << 3)
+                                           << (Levels() << 0 << 1 << 1 << 2);
+    QTest::newRow("keyval text, multiple words, no brace") << "\\mycommand{title=abc def}"
+                                 << (TTypes() << T::command << T::braces << T::keyVal_key<< T::word<< T::word)
+                                 << (STypes() << T::none << T::keyValArg << T::none<< T::text<< T::text)
+                                 << (Starts() << 0 << 10 << 11 << 17<< 21)
+                                 << (Length() << 10 << 15 << 5 << 3 << 3)
+                                 << (Levels() << 0 << 1 << 1 << 2 << 2);
+    QTest::newRow("keyval text in braces") << "\\mycommand{title={abc}}"
+                                   << (TTypes() << T::command << T::braces << T::keyVal_key<< T::braces << T::word)
+                                   << (STypes() << T::none << T::keyValArg << T::none<< T::text << T::text)
+                                   << (Starts() << 0 << 10 << 11 << 17 << 18)
+                                   << (Length() << 10 << 13 << 5 << 5 << 3)
+                                   << (Levels() << 0 << 1 << 1 << 3 << 3);
+    QTest::newRow("keyval text followed by second key/val") << "\\mycommand{title=abc,note={abc}}"
+                                                           << (TTypes() << T::command << T::braces << T::keyVal_key<< T::word<< T::keyVal_key<< T::braces << T::word)
+                                                           << (STypes() << T::none << T::keyValArg << T::none<< T::text<< T::none << T::keyVal_val << T::keyVal_val)
+                                                           << (Starts() << 0 << 10 << 11 << 17<< 21<<26<<27)
+                                                           << (Length() << 10 << 22 << 5 << 3 << 4<<5 <<3)
+                                                           << (Levels() << 0 << 1 << 1 << 2  <<1 << 3 << 3);
+    QTest::newRow("bibitem") << "\\bibitem{abc}"
+                                 << (TTypes() << T::command << T::braces << T::newBibItem)
+                                 << (STypes() << T::none << T::newBibItem << T::none)
+                                 << (Starts() << 0 << 8 << 9)
+                                 << (Length() << 8 << 5 << 3)
+                                 << (Levels() << 0 << 1 << 1);
 
 }
 
@@ -1051,10 +1099,30 @@ void LatexParsingTest::test_getContext_data() {
                             << 2
                             << (TTypes() << T::word)
                             << (STypes() << T::none);
+    QTest::newRow("none") << "abc   abc"
+                            << 4
+                            << (TTypes() )
+                            << (STypes());
     QTest::newRow("command") << "\\section{abc}"
                             << 10
                             << (TTypes() << T::command <<T::braces<<T::word)
                             << (STypes() << T::none << T::title<<T::title);
+    QTest::newRow("command, unclosed brace") << "\\section{abc"
+                             << 10
+                             << (TTypes() << T::command <<T::openBrace<<T::word)
+                             << (STypes() << T::none << T::title<<T::title);
+    QTest::newRow("command, unclosed brace with spaces") << "\\section{abc   "
+                             << 13
+                             << (TTypes() << T::command <<T::openBrace<<T::word)
+                             << (STypes() << T::none << T::title<< T::title);
+    QTest::newRow("after command") << "\\section{abc}  "
+                             << 14
+                             << (TTypes())
+                             << (STypes());
+    QTest::newRow("after command on word") << "\\section{abc} abc"
+                           << 16
+                           << (TTypes() << T::word)
+                           << (STypes() << T::none);
     QTest::newRow("command without braces") << "\\section abc"
                             << 10
                             << (TTypes() << T::command << T::word)
@@ -1063,14 +1131,18 @@ void LatexParsingTest::test_getContext_data() {
                             << 10
                             << (TTypes() << T::command << T::squareBracket<<T::word)
                             << (STypes() << T::none << T::shorttitle<<T::shorttitle);
+    QTest::newRow("command with optional arg 2") << "\\section[fds]{abc}"
+                             << 15
+                             << (TTypes() << T::command <<T::braces<<T::word)
+                             << (STypes() << T::none << T::title<<T::title);
     QTest::newRow("command with keyval") << "\\includegraphics[width=4cm]{abc}"
                             << 18
                             << (TTypes() << T::command << T::squareBracket<<T::keyVal_key)
                             << (STypes() << T::none << T::keyValArg<<T::none);
     QTest::newRow("command with keyval2") << "\\includegraphics[width=4cm]{abc}"
                             << 23
-                            << (TTypes() << T::command << T::squareBracket)
-                            << (STypes() << T::none << T::keyValArg);
+                            << (TTypes() << T::command << T::squareBracket << T::keyVal_key)
+                            << (STypes() << T::none    << T::keyValArg     << T::none);
                             //<< (TTypes() << T::command << T::squareBracket<<T::keyVal_key<<T::width) // this may be the desired outcome
                             //<< (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val);
     QTest::newRow("command with keyval3") << "\\includegraphics[width=4cm]{abc}"
@@ -1081,18 +1153,26 @@ void LatexParsingTest::test_getContext_data() {
                             << 28
                             << (TTypes() << T::command << T::squareBracket<<T::keyVal_key)
                             << (STypes() << T::none << T::keyValArg<<T::none);
+    QTest::newRow("command with keyval5") << "\\includegraphics[width as=4cm]{abc}"
+                          << 24
+                          << (TTypes() << T::command << T::squareBracket<<T::keyVal_key)
+                          << (STypes() << T::none << T::keyValArg<<T::none);
     QTest::newRow("command with empty keyval") << "\\includegraphics[width=]{abc}"  // #4017
                             << 23
                             << (TTypes() << T::command << T::squareBracket<<T::keyVal_key<<T::keyVal_val)
                             << (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val);
+    QTest::newRow("command with empty keyval and space") << "\\includegraphics[width= ]{abc}"  // #4218
+                           << 23
+                           << (TTypes() << T::command << T::squareBracket<<T::keyVal_key)
+                           << (STypes() << T::none << T::keyValArg<<T::none);
     QTest::newRow("command with keyval as defined argument") << "\\lstdefinelanguage{Excel}{morekeywords={ab$c}}"
-                                          << 41
-                                                             << (TTypes() << T::command << T::braces<<T::keyVal_key<<T::braces)
-                                                             << (STypes() << T::none << T::keyValArg<<T::none<<T::definition);
+                            << 41
+                             << (TTypes() << T::command << T::braces<<T::keyVal_key<<T::braces)
+                             << (STypes() << T::none << T::keyValArg<<T::none<<T::definition);
     QTest::newRow("command with keyval as label") << "\\lstdefinelanguage{Excel}{label=test}"
-                                                             << 35
-                                                             << (TTypes() << T::command << T::braces<<T::keyVal_key<<T::label)
-                                                             << (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val);
+                             << 35
+                             << (TTypes() << T::command << T::braces<<T::keyVal_key<<T::label)
+                             << (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val);
     QTest::newRow("following command") << "bummerang\\text" // command after word, #3967
                             << 9
                             << (TTypes() << T::word)
@@ -1103,9 +1183,6 @@ void LatexParsingTest::test_getContext_data() {
         << 23
         << (TTypes() << T::command << T::braces<<T::keyVal_key<<T::command<<T::braces<<T::bibItem)
         << (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val<<T::bibItem<<T::none);
-
-
-
 }
 
 void LatexParsingTest::test_getContext() {
@@ -1149,6 +1226,109 @@ void LatexParsingTest::test_getContext() {
     delete doc;
 }
 
+void LatexParsingTest::test_getContextMultiLine_data() {
+    QTest::addColumn<QString>("lines");
+    QTest::addColumn<int >("nr");
+    QTest::addColumn<int >("ln");
+    QTest::addColumn<TTypes>("desiredResults");
+    QTest::addColumn<STypes>("types");
+
+    QTest::newRow("simple") << "bummerang\nbummerang"
+                            << 2 << 1
+                            << (TTypes() << T::word)
+                            << (STypes() << T::none);
+    QTest::newRow("command") << "\\section{\nabc}"
+                             << 1 << 1
+                             << (TTypes() <<T::openBrace<<T::word)
+                             << (STypes() << T::title<<T::title);
+    QTest::newRow("command, unclosed brace") << "\\section{abc\nabc\n}"
+                             << 2 << 1
+                             << (TTypes() << T::openBrace<<T::word)
+                             << (STypes() << T::title<<T::title);
+    QTest::newRow("command, unclosed brace with spaces") << "\\section{abc\n   "
+                             << 1 << 1
+                             << (TTypes() << T::openBrace)
+                             << (STypes() << T::title);
+    QTest::newRow("after command") << "\\section{abc}\n  "
+                           << 1 << 1
+                           << (TTypes())
+                           << (STypes());
+    QTest::newRow("after command on word") << "\\section{abc}\n abc"
+                               << 2 << 1
+                               << (TTypes() << T::word)
+                               << (STypes() << T::none);
+    QTest::newRow("command with keyval") << "\\includegraphics[\nwidth=4cm]{abc}"
+                             << 1 << 1
+                             << (TTypes() << T::openSquare<<T::keyVal_key)
+                             << (STypes() << T::keyValArg<<T::none);
+    QTest::newRow("command with keyval2") << "\\includegraphics[\nwidth=4cm]{abc}"
+                              << 7 << 1
+                              << (TTypes() << T::openSquare << T::keyVal_key << T::width)
+                              << (STypes() << T::keyValArg     << T::none << T::keyVal_val);
+    QTest::newRow("command with keyval3") << "\\includegraphics\n[\nwidth=4cm]{abc}"
+                                         << 1 << 2
+                                         << (TTypes() << T::openSquare<<T::keyVal_key)
+                                         << (STypes() << T::keyValArg<<T::none);
+    QTest::newRow("command with keyval4") << "\\includegraphics\n[\nwidth=4cm]{abc}"
+                                  << 7 << 2
+                                  << (TTypes() << T::openSquare << T::keyVal_key << T::width)
+                                  << (STypes() << T::keyValArg     << T::none << T::keyVal_val);
+    QTest::newRow("command with keyval5") << "\\includegraphics\n[\nwidth=      ]{abc}"
+                                          << 7 << 2
+                                          << (TTypes() << T::openSquare<<T::keyVal_key)
+                                          << (STypes() << T::keyValArg<<T::none);
+    QTest::newRow("after command, open brace") << "\\section{\nabc}  "
+                                   << 5 << 1
+                                   << (TTypes())
+                                   << (STypes());
+}
+
+void LatexParsingTest::test_getContextMultiLine() {
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
+    LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
+    LatexPackage pkg_listings = loadCwlFile("txs-test.cwl");
+    lp->commandDefs.unite(pkg_listings.commandDescriptions);
+    QFETCH(QString,lines);
+    QFETCH(int, nr);
+    QFETCH(int, ln);
+    QFETCH(TTypes, desiredResults);
+    QFETCH(STypes, types);
+
+    QDocument *doc = new QDocument();
+    doc->setText(lines, false);
+    for(int i=0; i<doc->lines(); i++){
+        QDocumentLineHandle *dlh = doc->line(i).handle();
+        Parsing::simpleLexLatexLine(dlh);
+    }
+    TokenStack stack;
+    CommandStack commandStack;
+    for(int i=0; i<doc->lines(); i++){
+        QDocumentLineHandle *dlh = doc->line(i).handle();
+        Parsing::latexDetermineContexts2(dlh, stack, commandStack, lp);
+    }
+    if(doc->lineCount()>ln){
+        QDocumentLineHandle *dlh = doc->line(ln).handle();
+        //TokenList tl= dlh->getCookieLocked(QDocumentLine::LEXER_COOKIE).value<TokenList >();
+
+        TokenStack result = Parsing::getContext(dlh,nr);
+
+        for(int k=0;k<result.size();k++){
+            if(k>=desiredResults.size()){
+                continue;
+            }
+            QVERIFY2(result.at(k).type==desiredResults.at(k), QString("incorrect type at index %1:%2").arg(k).arg(lines).toLatin1());
+            QVERIFY2(result.at(k).subtype==types.at(k), QString("incorrect subtype at index %1:%2").arg(k).arg(lines).toLatin1());
+        }
+        QVERIFY2(result.size()==desiredResults.size(), QString("incorrect stacksize: %1:%2").arg(result.size()).arg(desiredResults.size()).toLatin1());
+    }else{
+        qDebug()<<"test case lines !";
+    }
+
+    delete doc;
+}
+
 void LatexParsingTest::test_getCompleterContext_data() {
     QTest::addColumn<QString>("lines");
     QTest::addColumn<int >("nr");
